@@ -10,6 +10,8 @@ import matplotlib as mpl
 from matplotlib.pyplot import figure
 from matplotlib.patches import Rectangle
 from sklearn import cluster, mixture
+import sklearn.linear_model as lm
+import sklearn.metrics as met
 
 from importlib_resources import files
 
@@ -470,4 +472,108 @@ def get_abbreviations( ):
             'Treg', 'MDSC_G', 'MDSC_M', 'Endo tip', 'Endo Lymp', 'Epi lum', 'Epi mam']
     rename_dict['celltype_subset'] = dict(zip(lst, lst2))
     
-    return rename_dict
\ No newline at end of file
+    return rename_dict
+
+
+def split_text( s, N_max = 20 ):
+    
+    if len(s) > N_max:
+        positions = [i for i, c in enumerate(s) if c == ' ']
+        positions.append(len(s) - 1)
+        pcnt = 0
+        ps = []
+        pc = 0
+        for j, p in enumerate(positions):
+            if (j > 0) & ((p - pc) > N_max):
+                ps.append(positions[j-1]+1)
+                pc = positions[j-1]+1
+
+        if len(ps) > 0:
+            s2 = s
+            for p in reversed(ps):
+                s2 = s2[:p] + '\n' + s2[p:]
+            return s2
+        else:
+            return s
+    else:
+        return s
+
+
+def evaluate_performance(y, y_pred, labels = ['Tumor', 'Normal'] ):
+
+    acc = np.sum(y == y_pred)/len(y)
+    # print('Acc: %4.2f ' % acc)
+    perf = {}
+    perf['Accuracy'] = np.round(acc,3)
+    perf['F1'] = 0
+    f1_all = []
+    nc = []
+    for t in labels:
+        b1 = y == t
+        b2 = y_pred == t
+        bc = b1 & b2
+        prec = np.sum(bc)/np.sum(b2)
+        recall = np.sum(bc)/np.sum(b1)
+        bd = (~b1) & b2
+        fpr = np.sum(bd)/np.sum(~b1)
+        f1 = 2*prec*recall/(prec + recall)
+        perf['F1 (%s)' %t] = np.round(f1,3)
+        perf['Precision (%s)' %t] = np.round(prec,3)
+        perf['Recall (%s)' %t] = np.round(recall,3)
+        perf['FPR (%s)' %t] = np.round(fpr,3)
+        f1_all.append(f1)
+        nc.append(np.sum(b1))
+    perf['F1'] = np.round(np.sum(np.array(f1_all)*np.array(nc))/np.sum(nc), 3)
+    
+    return perf
+
+
+def compute_sample_div_index( adata, sample_col = 'sample',
+                              cluster_col = 'cnv_cluster',
+                              tumor_dec = 'ploidy_dec',
+                              ref_ind_col = 'cnv_ref_ind',
+                              sdi_col = 'sample_diversity_index',
+                              tumor_name = 'Aneuploid'):
+
+    slst = adata.obs[sample_col].unique().tolist()
+    clust_lst = adata.obs[cluster_col].unique().tolist()
+    clust_lst.sort()
+    
+    df = pd.DataFrame( columns = ['Tumor_pct', 'N_samples', 'sample_diversity_index'], 
+                       index = clust_lst )
+    for c in clust_lst:
+        b = adata.obs[cluster_col] == str(c)
+        pcnt = adata.obs.loc[b, sample_col].value_counts()
+        pcnt = pcnt/np.sum(pcnt)
+        pcnt = pcnt[pcnt > 0]
+        H = np.sum( -pcnt*np.log2(pcnt) )    
+        pt = int( 100*(adata.obs.loc[b, tumor_dec] == tumor_name).sum()/np.sum(b) )
+        pt_ref = int( 100*(adata.obs.loc[b, ref_ind_col]).sum()/np.sum(b) )
+        Ns = np.sum(pcnt > 0)
+        # print('%2i: p_t: %i, n_s: %i -> H = %4.2f' % (c, pt, np.sum(pcnt > 0), 2**H) )
+    
+        df.loc[c, 'Tumor_pct'] = pt
+        df.loc[c, 'Ref_pct'] = pt_ref
+        df.loc[c, 'N_samples'] = Ns
+        df.loc[c, 'sample_diversity_index'] = H
+    
+    df['Tumor'] = df['Tumor_pct'] >= 50
+
+    adata.obs[sdi_col] = 0
+    for c in df.index.values.tolist():
+        b = adata.obs[cluster_col] == str(c)
+        adata.obs.loc[b, sdi_col] = df.loc[c, 'sample_diversity_index'].round()
+    adata.obs[sdi_col] = adata.obs[sdi_col].astype(str)
+    
+    return df
+
+
+def get_axes( axes, nr, nc, j ):
+    if nr > 1:
+        jr = int(np.floor(j/nc))
+        jc = j%nc
+        ax = axes[jr][jc]
+    else:
+        ax = axes[j]
+    return ax
+    
\ No newline at end of file
